# Useful GPGPU Programming Abstractions - Appendix A
This is an appendix to Johannes Larsen's [master's
thesis](https://static.johslarsen.net/uit/master-thesis/) at the Arctic
University of Norway (UiT).

## Content
* `src/common`: source code shared between benchmarks, see that for details
* `src/clustering`: compute-intensive terrain clustering benchmark, see that for details
* `src/streaming_pipeline`: data-intensive feature extraction benchmark, see that for details

* `share/benchmark_streaming_pipeline/20160528_163422*`: benchmarking results for Figure 4.4a
* `share/benchmark_streaming_pipeline/20160528_195138*`: benchmarking results for Figure 4.4b
* `share/benchmark_clustering/20160531_230806*`: benchmarking results for Figure 4.5
* `share/benchmark_clustering/20160531_144401*`: benchmarking results for Figure 4.6
* `share/profile_clustering/20160528_143931*`: profiling results for Figure 5.1
* `share/profile_streaming_pipeline/20160530_164051*`: profiling results for Figure 5.2
* `share/profile_clustering/20160530_172603*`: profiling results for Figure 5.3

* `share/plot/spBlocksize.*`: Figure 4.4a
* `share/plot/spWindow.*`: Figure 4.4b
* `share/plot/cSubsampleFixedWorkload.*`: Figure 4.6
* `share/plot/cBreakdown.*`: Figure 5.1
* `share/plot/spMemoryBytes.*`: Figure 5.2a
* `share/plot/spMemoryBreakdown.*`: Figure 5.2b
* `share/plot/cMemoryBytes.*`: Figure 5.3a
* `share/plot/cMemoryBreakdown.*`: Figure 5.3b

## Benchmarking
The benchmarks are run with `src/common/run_all_parameterized.rb` from the
benchmarks' source code directories. And the experiments were:
* Figure 4.4a: `../common/run_all_parameterized.rb -o ../../share/benchmark_streaming_pipeline/ NCOL_PER_CHUNK=25,100:1000:100 WINDOW=25`
* Figure 4.4b: `../common/run_all_parameterized.rb -o ../../share/benchmark_streaming_pipeline/ NWINDOW=5,10:100:10 NCOL_PER_CHUNK=300`
* Figure 4.5: `../common/run_all_parameterized.rb -o ../../share/benchmark_clustering/ NSUBSAMPLE=50,55,60,66,75,85,100,120,150,200,300,600,3000`
* Figure 4.6: `../common/run_all_parameterized.rb -o ../../share/benchmark_clustering/ NSUBSAMPLE=50,55,60,66,75,85,100,120,150,200,300,600,3000 {MIN,MAX}_CLASS=10 {MIN,MAX}_EM_ITERATIONS=100`

## Profiling

### Clustering Hot-Spot
The native implementations are profiled with: `nvprof --profile-child-processes
--cpu-profiling on --cpu-profiling-frequency 500 --cpu-profiling-mode top-down
<WORKER> <CMX>...` ArrayFire Python is profiled with: `python3 -m cProfile -s
cumtime worker_af.py <CMX>...` MATLAB is profiled by manually adding `profile
on` in the beginning and `profsave` in the end, and running it manually.

The data files are manually written, and they contain the relative breakdown of
the `split_until_all_classes_within_confidence_interval` function. Grouping:

* MoGEM: `MoGEM`
* Score: `score` + `determine_least_fitting_class`(this would be the synchronization point)
* Split: `split_class`
* Other: total - the others

### Memory

All, but MATLAB implementations, are profiled with: `nvprof --print-gpu-trace
<WORKER> <CMX>... | ../common/aggregate_memory_trace.awk`. MATLAB uses child
processes, which are not compatible with `--gpu-trace`, so the actual command
line is determined using `ps aux | grep -i matlab`, and replaces `<WORKER>
<CMX>...` in the `nvprof` statement.

I do not bother storing the huge GPU trace output, and in the aggregated output
the 2th column is the category, 4th column is bytes, 5th column is aggregated
delay and 6th column is total time, and other is the total time minus the other
categories.





## Data plotting
Plot data is generated by `dat_from_benchmark_output.sh <BENCHMARK_OUTPUT>...`
in the respective benchmark's source code directory, but note that these
scripts reference particular data files, so they must be manually edited for
plotting other results.
