#!/bin/gnuplot
set term post landscape size 14.8cm,21cm color enhanced
set output "spBlocksize.ps"
set multiplot

set title "Streaming pipeline performance, l_x = l_y = 25";
set colorsequence podo;
set linetype cycle 32;
set arrow from (300*19431),graph 0 to (300*19431),graph 1 nohead linetype 0;
set xrange [0:*];
set ylabel "Runtime (s)";

set label 1 center at graph .5,0.9 "Full range"
set tmargin at screen 0.95;
set bmargin at screen 0.75;
set lmargin at screen 0.15;
set ytics 20,20,120;
set yrange [0:*];
unset xtics;
set key off;
plot\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_af_cuda" using ($2*19431):3:5:6 with errorlines title "ArrayFire CUDA",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_af.py" using ($2*19431):3:5:6 with errorlines title "ArrayFire Python",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_cuda" using ($2*19431):3:5:6 with errorlines title "CUDA",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_matlab.sh" using ($2*19431):3:5:6 with errorlines title "MATLAB",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_matlab_cpu.sh" using ($2*19431):3:5:6 with errorlines title "MATLAB (CPU 16 cores)",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_cuda.py" using ($2*19431):3:5:6 with errorlines title "PyCUDA",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_openacc" using ($2*19431):3:5:6 with errorlines title "OpenACC",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_thrust" using ($2*19431):3:5:6 with errorlines title "Thrust",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_vexcl_cuda" using ($2*19431):3:5:6 with errorlines title "VexCL CUDA",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_vexcl_cl" using ($2*19431):3:5:6 with errorlines title "VexCL OpenCL",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_af_cl" using ($2*19431):3:5:6 with errorlines title "ArrayFire OpenCL",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_openmp" using ($2*19431):3:5:6 with errorlines title "OpenMP (CPU 16 cores)";

set notitle;
set xlabel "n_{sample per block}";
set label 1 center at graph .5,0.95 "Zoomed"
set key right bottom;
set tmargin at screen 0.75;
set bmargin at screen 0.10;
set yrange [0:12];
set ytics auto;
set xtics 1e6,3e6,1e8;
plot\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_af_cuda" using ($2*19431):3:5:6 with errorlines title "ArrayFire CUDA",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_af.py" using ($2*19431):3:5:6 with errorlines title "ArrayFire Python",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_cuda" using ($2*19431):3:5:6 with errorlines title "CUDA",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_matlab.sh" using ($2*19431):3:5:6 with errorlines title "MATLAB",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_matlab_cpu.sh" using ($2*19431):3:5:6 with errorlines title "MATLAB (CPU 16 cores)",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_cuda.py" using ($2*19431):3:5:6 with errorlines title "PyCUDA",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_openacc" using ($2*19431):3:5:6 with errorlines title "OpenACC",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_thrust" using ($2*19431):3:5:6 with errorlines title "Thrust",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_vexcl_cuda" using ($2*19431):3:5:6 with errorlines title "VexCL CUDA",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_vexcl_cl" using ($2*19431):3:5:6 with errorlines title "VexCL OpenCL",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_af_cl" using ($2*19431):3:5:6 with errorlines title "ArrayFire OpenCL",\
		"../../share/benchmark_streaming_pipeline/20160528_163422_worker_openmp" using ($2*19431):3:5:6 with errorlines title "OpenMP (CPU 16 cores)";
