#pragma once

#include <cublas_v2.h>

class CudaMoGEM {
protected:
	cublasHandle_t ch;
	cudaStream_t cs;
public:
	unsigned nfeature, nsample, nclass;
	CudaMoGEM(unsigned nfeature, unsigned nsample, unsigned nclass, cublasHandle_t ch = NULL);

	void initilize_GMDistribution(float *P, float *mu, float *Sigma, const float *samples, const float *ones, float *tmpVar);
	void print_gm(const float *P, const float *mu, const float *Sigma) const;
	void classwise_share(float *shares, const float *weights, const float *ones, float *P_to_be_summed, float scalar=1.0);
	void pdfs_from_samples(float *pdfs, const float *samples, const float *P, const float *mu, float *luSigma, float *iSigma, int *pSigma, int *dbatchStatus, float **drefs);
	void weights_from_pdfs(float *weights, const float *pdfs);
	void mstep(float *P, float *mu, float *Sigma, const float *samples, const float *weights, float *FSC_A, float *FSC_B, float *FSC_C, const float *ones, const float *fallback_covariance);
	void score(float *scores, const float *mu, const float *Sigma, const float *samples, const float *weights, float *P_with_respect_to_weights, float *bin_bounds, float *weight_bins, float *P_to_be_summed, const float *ones);
	void determine_least_fitting_class(unsigned *hout_iworst_score, float *hout_worst_score, const float *scores, unsigned *index, float *value);
	void split_class(float *nP, float *nmu, float *nSigma, const float *P, const float *mu, const float *Sigma, unsigned iscore, float score);
	float log_likelihood(const float *pdfs, float *partial_aggregates);
};
