#include <cassert>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>

#include "CudaMoGEMAfBindings.hpp"
#include "common/af_utils.hpp"

enum {
	NSAMPLE_PER_CHUNK = 1<<18,
};

int main(int argc, char **argv)
{
	assert(argc >= 4);
	af::array mu = af_utils::load_cmx(argv[1]);
	af::array P = af_utils::load_cmx(argv[2]);
	af::array Sigma = af_utils::load_cmx(argv[3]);
	unsigned nfeature = mu.dims(0), nclass = mu.dims(1);
	CudaMoGEMAfBindings(nfeature,0,nclass).print_gm(P,mu,Sigma);
	argv+=4, argc-= 4;
	assert(argc = nfeature);

	af::dim4 dims = af_utils::dims_from_cmx(argv[0]);
	af::dtype T = af_utils::dtype_from_cmx(argv[0]);
	int *fdIn = (int *)alloca(nfeature * sizeof(*fdIn));
	char **feature = (char **)alloca(nfeature * sizeof(*feature));
	af::array samples(NSAMPLE_PER_CHUNK, nfeature, T);
	size_t nbyte_element = samples.bytes()/samples.elements();


	for (unsigned i=0; i < nfeature; i++) {
		std::cout << argv[i] << std::endl;
		fdIn[i] = open(argv[i], O_RDONLY); assert(fdIn[i] >= 0);
		feature[i] = (char *)mmap(NULL, nbyte_element*dims.elements(), PROT_READ, MAP_PRIVATE, fdIn[i], 0); assert(feature[i] != MAP_FAILED);
	}

	std::string fnOut = "labels" + af_utils::cmx_suffix(u8, dims);
	int fdOut = open(fnOut.c_str(), O_RDWR|O_CREAT|O_TRUNC, 0666); assert(fdOut >= 0);
	lseek(fdOut, dims.elements()-1, SEEK_SET);
	char a_byte = 0; write(fdOut, &a_byte, 1);
	char *labels = (char *)mmap(NULL, dims.elements(), PROT_WRITE, MAP_SHARED, fdOut, 0); assert(labels != MAP_FAILED);

	for (dim_t to=0, from=0; to < dims.elements(); from += NSAMPLE_PER_CHUNK) {
		to = from + std::min<dim_t>(NSAMPLE_PER_CHUNK, dims.elements()-from);
		std::cout << from << "/" << dims.elements() << std::endl;
		dim_t nsample = to-from;

		af::array samples(nfeature, nsample, T);
		for (dim_t i=0; i < nfeature; i++) {
			af::array this_feature(1, nsample, T);
			af_utils::writeDuckTyped(this_feature, feature[i] + nbyte_element*from, this_feature.bytes());
			samples(i, af::span) = this_feature;
		}

		CudaMoGEMAfBindings moGEM(nfeature,nsample,nclass);

		af::array pdfs(nclass,nsample), weights(nclass,nsample);
		moGEM.pdfs_from_samples(pdfs, samples, P, mu, Sigma);
		moGEM.weights_from_pdfs(weights, pdfs);

		af::array ignore, labeled;
		af::max(ignore, labeled, weights, 0);
		af_utils::hostDuckTyped(labeled.as(u8), labels + from, labeled.elements()*sizeof(uint8_t));
	}

	assert(msync(labels, dims.elements(), MS_SYNC) == 0);
	assert(munmap(labels, dims.elements()) == 0);

	return 0;
}
