#include <cstdio>
#include <ctime>
#include <iostream>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>

#include <arrayfire.h>
#include <cuda_runtime.h>

#include "backend.hpp"
#include "common/af_utils.hpp"
#include "common/monotonic.hpp"
#include "config.hpp"

using namespace af;

const char *me = NULL;

void write_to_disk(const float *harray, const std::string& prefix, const af::dim4& dims) {
	std::string filename = prefix + af_utils::cmx_suffix(f32, dims);

	int fd = open(filename.c_str(), O_RDWR|O_TRUNC|O_CREAT, 0666);
	ssize_t nbyte = dims.elements() * sizeof(*harray);
	assert(fd >= 0);
	assert(write(fd, harray, nbyte) == nbyte);
	assert(close(fd) == 0);
}

int main(int argc, char **argv) {
	monotonic::init();
	me = *(argc--,argv++);
	assert(argc > 0);

	dim4 original_dims = af_utils::dims_from_cmx(argv[0]);
	assert(af_utils::dtype_from_cmx(argv[0]) == f32);

	unsigned nfeature=argc, nsample=original_dims.elements()/SUBSAMPLE;
	float *hsamples = (float*)malloc(nfeature*nsample*sizeof(*hsamples));
	for (int i=0; i<argc; i++) {
		int fd = open(argv[i], O_RDWR); assert(fd >= 0);
		float *samples_original = (float *)mmap(NULL, original_dims.elements()*sizeof(*samples_original), PROT_READ, MAP_PRIVATE, fd, 0); assert(samples_original != MAP_FAILED);

		for (unsigned j=0; j<nsample; j++) {
			hsamples[j*nfeature + i] = samples_original[j*SUBSAMPLE];
		}
		assert(munmap(samples_original, original_dims.elements()*sizeof(*samples_original)) == 0);
		assert(close(fd) == 0);
	}


	float *hP = (float*)alloca(MAX_CLASSES*sizeof(*hP));
	float *hmu = (float*)alloca(nfeature*MAX_CLASSES*sizeof(*hmu));
	float *hSigma = (float*)alloca(nfeature*nfeature*MAX_CLASSES*sizeof(*hSigma));

	unsigned nclass;
	double tAMoG = monotonic::since();
	nclass = do_clustering(hP, hmu, hSigma, hsamples, nfeature, nsample);
	std::printf("%-10s %10.6lf %10.6lf\n", "AMOG", tAMoG, monotonic::since());

	write_to_disk(hP, "classifier_P", af::dim4(nclass));
	write_to_disk(hmu, "classifier_mu", af::dim4(nfeature,nclass));
	write_to_disk(hSigma, "classifier_Sigma", af::dim4(nfeature,nfeature,nclass));

	return 0;
}
