#pragma once
#include <cassert>
#include <cstring>
#include <cstdlib>
#include <ostream>
#include <string>
#include <arrayfire.h>
#include <fcntl.h>

namespace af_utils {
	struct Geometry {
		af::dim4 dims;
		af::dim4 offset;

		Geometry(af::dim4 dims, af::dim4 offset): dims(dims), offset(offset) {}
		Geometry(): dims(0,0,0,0), offset(0,0,0,0) {} // wannabe sentinel is empty
		Geometry(const char *s): dims(1,1,1,1), offset(0,0,0,0) {
			const char *next = s;
			bool next_is_offset = false;
			for (int id=0, io=0; *next != '\0';) {
				dim_t n = std::strtoull(next, (char **)&next, 10);
				assert(next != NULL);
				if (next_is_offset && io < 4) offset[io++] = n;
				else if (id < 4)  dims[id++] = n;
				else assert("TOO MANY DIMS/OFFSETS" == NULL);

				switch (*next) {
					case '+': next_is_offset = true; // fallthrough
					case 'x': next++; break;
					case '\0': break;
					default: assert("INVALID GEOMETRY SEPARATOR" == NULL);
				}
			}
		}
		Geometry(const std::string &s): Geometry(s.c_str()) {}
	};


	bool dim4_div(const af::dim4 &dividend, const af::dim4 &divisor, af::dim4 *quotient, af::dim4 *remainder);
	bool dim4_le(const af::dim4 &a, const af::dim4 &b);
	af::dim4 dim4_parse(const char *decimalnumbers, dim_t filler);
	af::dim4 dim4_parse(const std::string &decimalnumbers, dim_t filler);

	af::array crop(const af::array &A, const Geometry &g);
	af::array cropToDivide(const af::array &in, const af::dim4 &divisor);

	af::array normalize(const af::array &A, double new_min=0.0, double new_max=1.0);
	af::array clampAbove(const af::array &A, double upperThreshold);
	af::array clampBelow(const af::array &A, double lowerThreshold);
	af::array clamp(const af::array &A, double lowerThreshold, double upperThreshold);
	af::array clampStatistically(const af::array &A, double sigmaBelow=-3.0, double sigmaAbove=3.0);
	void update(af::array &A, const af::array &filter, double filler, dim_t *filterCount=NULL);
	af::array replace(const af::array &A, const af::array &filter, double filler, dim_t *filterCount=NULL);
	af::array replaceNaN(const af::array &A, double filler, dim_t *nNaN=NULL);
	af::array replaceInf(const af::array &A, double filler, dim_t *nInf=NULL);
	af::array nonInfNaN(const af::array &A, double filler=0.0, dim_t *nInf=NULL, dim_t *nNaN=NULL);

	af::array maximizeRange(const af::array &A);
	af::dim4 squeeze(const af::dim4& dims);
	af::array squeeze(const af::array &A);

	void writeDuckTyped(af::array &A, void *ptr, const size_t bytes, af::source src=afHost);
	void hostDuckTyped(const af::array &A, void *ptr, const size_t bytes);
	af::array load_cmx(const char *filename);
	af::array load_cmx(const std::string &filename);
	void store_cmx(const af::array &A, const char *filename_without_cmx_suffix, int oflag=O_RDWR|O_CREAT|O_TRUNC, mode_t mode=0666);
	void store_cmx(const af::array &A, const std::string &filename_without_cmx_suffix, int oflag=O_RDWR|O_CREAT|O_TRUNC, mode_t mode=0666);

	af::dtype dtype_parse(const char *str, size_t n=4); // default n: longest dtype
	af::dtype dtype_parse(const std::string &filename);
	af::dtype dtype_from_cmx(const char *filename);
	af::dtype dtype_from_cmx(const std::string &filename);
	af::dim4 dims_from_cmx(const char *filename);
	af::dim4 dims_from_cmx(const std::string &filename);
	std::string cmx_suffix(af::dtype type, const af::dim4 &dims);
	std::string cmx_suffix(const af::array &A);

	af::array cov_rowwise(const af::array &A, const af::array &A_rowwise_mu);
	af::array cov_rowwise(const af::array &A);
	af::array cov_colwise(const af::array &A, const af::array &A_colwise_mu);
	af::array cov_colwise(const af::array &A);
	af::array clog(const af::array &Z);
	bool issigned(const af::array &A);
	bool isfloat(const af::array &A);

	void dump_info(std::ostream &out, const af::array &A, const std::string &name="A");

	extern const char *dtype_names[];
}
