#pragma once

#include <cassert>

#include <thrust/iterator/counting_iterator.h>
#include <thrust/iterator/transform_iterator.h>
#include <thrust/iterator/permutation_iterator.h>
#include <thrust/functional.h>

// derived from Peter Wittek's example (CC-BY-NC):
//
//   * http://peterwittek.com/summing-the-entries-of-a-matrix-using-a-stencil-with-thrust.html (blog article)
//   * https://gist.github.com/peterwittek/6303588 (source code)

template<typename Iterator>
class column_major_2d_submatrix_iterator {
public:
	typedef typename thrust::iterator_difference<Iterator>::type T_n;

	struct submatrix_functor : public thrust::unary_function<T_n,T_n> {
		T_n ny, nY;

		submatrix_functor(T_n ny_submatrix, T_n nY_supermatrix):
			ny(ny_submatrix), nY(nY_supermatrix) {}

		__host__ __device__ T_n operator()(const T_n& i) const {
			int x = i/ny, y = i%ny;
			return nY*x + y;
		}
	};

	column_major_2d_submatrix_iterator(Iterator first, Iterator last, T_n nx_submatrix, T_n ny_submatrix, T_n nY_supermatrix):
		first(first), nx(nx_submatrix), ny(ny_submatrix), nY(nY_supermatrix) {
			assert(ny <= nY);
			assert(first + nY*(nx-1) + ny <= last);
	}

	typedef typename thrust::counting_iterator<T_n> Counter;
	typedef typename thrust::transform_iterator<submatrix_functor, Counter> Transformer;
	typedef typename thrust::permutation_iterator<Iterator, Transformer> Permutator;

	Permutator begin(void) const {
		return Permutator(first, Transformer(Counter(0), submatrix_functor(ny, nY)));
	}
	Permutator end(void) const {
		return begin() + nx*ny;
	}

protected:
	Iterator first;
	T_n nx, ny, nY;
};
