#pragma once

#include <cuComplex.h>

#include <cstdlib>
#include <iostream>

#include "cuda.hpp"

#define BS_1D dim3(256)
#define BS_2D dim3(16,16)

static inline unsigned iceil_div(unsigned dividend, unsigned divisor) {
	return (dividend + (divisor-1)) / divisor;
}
static inline dim3 gs(dim3 p, dim3 bs) {
	return dim3(iceil_div(p.x,bs.x), iceil_div(p.y,bs.y), iceil_div(p.z,bs.z));
}

__host__ static inline std::ostream& operator<<(std::ostream& ostr, const dim3& dims) {
	ostr << dims.x << " " << dims.y << " " << dims.z;
	return ostr;
}

template <typename T>
__global__ void set_array(dim3 span, T *array, T value) {
	for (int i = blockIdx.x*blockDim.x + threadIdx.x; i < span.x; i += blockDim.x*gridDim.x) {
		array[i] = value;
	}
}
template <typename T>
__global__ void scale_array(dim3 span, T *out, const T *in, T scalar) {
	for (int i = blockIdx.x*blockDim.x + threadIdx.x; i < span.x; i += blockDim.x*gridDim.x) {
		out[i] = scalar * in[i];
	}
}

__device__ __host__ static inline cuFloatComplex makeFromDouble(double a, cuFloatComplex ignore) { return make_cuFloatComplex(float(a), 0.0); }
__device__ __host__ static inline cuFloatComplex operator*(cuFloatComplex a, cuFloatComplex b) { return cuCmulf(a,b); }
__device__ __host__ static inline cuFloatComplex operator/(cuFloatComplex a, cuFloatComplex b) { return cuCdivf(a,b); }
__device__ __host__ static inline cuFloatComplex operator+(cuFloatComplex a, cuFloatComplex b) { return cuCaddf(a,b); }
__device__ __host__ static inline cuFloatComplex operator-(cuFloatComplex a, cuFloatComplex b) { return cuCsubf(a,b); }
__device__ __host__ static inline cuFloatComplex fma(cuFloatComplex x, cuFloatComplex y, cuFloatComplex d) { return cuCfmaf(x, y, d); }
__device__ __host__ static inline double abs(cuFloatComplex a) { return cuCabsf(a); }
__device__ __host__ static inline cuFloatComplex log(cuFloatComplex z) {
	return make_cuFloatComplex(abs(z), atan2f(cuCimagf(z), cuCrealf(z)));
}

__device__ __host__ static inline cuDoubleComplex makeFromDouble(double a, cuDoubleComplex ignore) { return make_cuDoubleComplex(a, 0.0); }
__device__ __host__ static inline cuDoubleComplex operator*(cuDoubleComplex a, cuDoubleComplex b) { return cuCmul(a,b); }
__device__ __host__ static inline cuDoubleComplex operator/(cuDoubleComplex a, cuDoubleComplex b) { return cuCdiv(a,b); }
__device__ __host__ static inline cuDoubleComplex operator+(cuDoubleComplex a, cuDoubleComplex b) { return cuCadd(a,b); }
__device__ __host__ static inline cuDoubleComplex operator-(cuDoubleComplex a, cuDoubleComplex b) { return cuCsub(a,b); }
__device__ __host__ static inline cuDoubleComplex fma(cuDoubleComplex x, cuDoubleComplex y, cuDoubleComplex d) { return cuCfma(x, y, d); }
__device__ __host__ static inline double abs(cuDoubleComplex a) { return cuCabs(a); }
__device__ __host__ static inline cuDoubleComplex log(cuDoubleComplex z) {
	return make_cuDoubleComplex(abs(z), atan2f(cuCimag(z), cuCreal(z)));
}

__device__ __host__ static inline double makeFromDouble(double a, double ignore) { return a; }
__device__ __host__ static inline float makeFromDouble(double a, float ignore) { return float(a); }
