#!/usr/bin/env ruby
require 'fileutils'
require 'optparse'
require 'shellwords'
require 'time'

$options = {
  prefix: '../share/benchmark/',
  suffix: '.output',
  config: 'config.py',
  cmd: ['make', '-e', '-j'+`nproc`.strip, 'run_all'],
}
OptionParser.new do |o|
  o.banner = "Usage: #{$0} [$options] <KEY=FROM[:INCLUSIVE_TO[:STEP]]>[,...]>... [== CMDLINE=#{$options[:cmd].join(' ')}"

  o.on "-c", "--config FILE", "What configuration file to modify, default: #{$options[:config]}" do |f|
    $options[:config] = f
  end
  o.on "-o", "--output PREFIX", "Where to write output files, default: #{$options[:prefix]}" do |p|
    $options[:prefix] = p
  end
  o.on "-s", "--suffix SUFFIX", "Output filesuffix, default: #{$options[:suffix]}" do |s|
    $options[:suffix] = s
  end
end.parse!

def generate_permutations(base, args)
  if args[0] == "=="
    $options[:cmd] = args[1..-1]
    args=[]
  end
  if args.empty?
    return [base]
  end

  permutations = []

  key,ranges = args[0].split("=")
  ranges.split(",").each do |r|
    from, to, increment = r.split(":")
    unless to.nil?
      if from =~ /[.eE]/
        iterator = Range.new(from.to_f, to.to_f).step(increment.nil? ? 1 : increment.to_f)
      else
        iterator = Range.new(from.to_i, to.to_i).step(increment.nil? ? 1 : increment.to_i)
      end

      iterator.each do |i|
        permutations.concat generate_permutations(base+[[key,i.to_s]], args[1..-1])
      end
    else
      permutations.concat generate_permutations(base+[[key,from]], args[1..-1])
    end
  end
  return permutations
end
benchmarks = generate_permutations [], ARGV #"#{base}_#{key}=#{value.sub(/[[:space:]]/, "_")}", args

start = Time.now
sha = `git rev-parse HEAD`.strip

FileUtils.cp $options[:config], "#{$options[:config]}.#{start.strftime "%Y%m%d_%H%M%S"}"
config = File.readlines($options[:config]).join()

benchmarks.shuffle.each_with_index do |b, i|
  permutation = b.map{|kv| "_#{kv[0]}=#{kv[1].sub(/[[:space:]]/, "_")}"}.join("")
  output = "#{$options[:prefix]}#{start.strftime "%Y%m%d_%H%M%S"}_#{sha}#{permutation}#{$options[:suffix]}"

  b.each do |kv|
    k,v = kv
    config.sub!(/^[[:space:]]*#{k}[[:space:]]*=.*$/, "#{k}=#{v}")
    ENV[k]=v
  end
  File.open($options[:config], 'w') do |f|
    f.write(config)
  end

  puts "%12.3f %4d/%d: #{output}"%[Time.now-start, i, benchmarks.length]
  `#{Shellwords.join $options[:cmd]} > #{Shellwords.escape output}`
end
