#pragma once

#include "common.cuh"
#include <cublas_v2.h>

template<typename T>
__global__ static void variance_rowwise(dim3 problem, T *Var, int Var_stride, const T *X, int X_stride, const T *mu, int mu_stride) {
	unsigned i = blockIdx.x*blockDim.x + threadIdx.x;
	unsigned nrow = problem.x, nsample = problem.y;
	if (i >= nrow) return;
	for (unsigned j = blockIdx.y*blockDim.y + threadIdx.y; j < nsample; j += blockDim.y*gridDim.y) {
		Var[j*nrow*Var_stride + i*Var_stride] = X[j*nrow*X_stride + i*X_stride] - mu[i*mu_stride];
	}
}
static inline void cov_rowwise(cublasHandle_t ch, float *Sigma, float *tmp_Var, const float *X, dim3 sX, const float *mu, int Sigma_stride=1, int Var_stride=1, int X_stride=1, int mu_stride=1) {
	cudaStream_t stream;
	CUDA_DIE_UNLESS_SUCCESS(cublasGetStream(ch, &stream));

	unsigned tx = std::min((unsigned)sX.x, (unsigned)16);
	dim3 threads(tx, 256/tx);
	variance_rowwise<<<gs(sX,threads),threads,0,stream>>>(sX, tmp_Var, Var_stride, X, X_stride, mu, mu_stride);

	float alpha=1.0/sX.y, beta=0.0;
	CUDA_DIE_UNLESS_SUCCESS(cublasSgemm(ch, CUBLAS_OP_N, CUBLAS_OP_T, sX.x, sX.x, sX.y, &alpha, tmp_Var, Var_stride*sX.x, tmp_Var, Var_stride*sX.x, &beta, Sigma, Sigma_stride*sX.x));
}
