#include <cassert>
#include <cstring>
#include <fstream>

#include <arrayfire.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>

#include "../af_geotiff.hpp"
#include "../af_utils.hpp"

af::dtype output_type = (af::dtype)-1;

char *me;
static void die(const std::string &error = "") {
	if (!error.empty()) {
		std::cerr << "Error: " << error << std::endl;
	}
	std::cout << "Usage: "<<me<<" [-t DTYPE] <*.tiff>..." << std::endl;
	std::exit(error.empty() ? EXIT_SUCCESS : EXIT_FAILURE);
}

int main(int argc, char *argv[]) {
	me = argv[0];
	af::setBackend(AF_BACKEND_CPU); // lots of RAM

	while (--argc >= 0 && **(++argv) == '-') {
		switch(argv[0][1]) {
			case 'h':
			case '?':
				die();

			default:
				assert(argc >= 2);
				switch(argv[0][1]) {
					case 't': output_type = af_utils::dtype_parse(*(--argc,++argv)); break;

					default:
							 die("Invalid option: \""+std::string(argv[0])+"\"");
				}
		}
	}

	for (int i=0; i<argc; i++) {
		char *filenameIn = argv[i];
		std::cout << filenameIn << ": ";

		af::array img = af_geotiff::load(filenameIn);
		std::cout << af_utils::dtype_names[img.type()] << ", " << img.dims() << std::endl;
		bool org_is_complex = img.dims(3) == 2;
		if (org_is_complex) {
			af::array real = img(af::span,af::span,af::span,0);
			af::array imag = img(af::span,af::span,af::span,0);
			img = af::complex(real, imag);
		}
		if (output_type >= 0) {
			if (org_is_complex && output_type != c32 && output_type != c64) {
				img = abs(img);
			}
			img = img.as(output_type);
		}

		char *suffix = std::strrchr(filenameIn, '.');
		std::string base(filenameIn, suffix-filenameIn);
		std::string filenameOut = base + af_utils::cmx_suffix(img.type(), af::dim4(img.dims(0), img.dims(1), img.dims(2)));
		int fd = open(filenameOut.c_str(), O_RDWR|O_TRUNC|O_CREAT, 0666);
		assert(fd >= 0);
		char a_byte=0;
		lseek(fd, img.bytes()-1, SEEK_SET);
		write(fd, &a_byte, 1);
		void *io = mmap(NULL, img.bytes(), PROT_WRITE, MAP_SHARED, fd, 0);
		assert(io != MAP_FAILED);

		af_utils::hostDuckTyped(img, io, img.bytes());

		assert(munmap(io, img.bytes()) == 0);
		assert(close(fd) == 0);
	}

	return 0;
}
