#include <cassert>
#include <cstring>
#include <cstdlib>
#include <arrayfire.h>

#include "../af_utils.hpp"

char *me;
static void die(const std::string &error = "") {
	if (!error.empty()) {
		std::cerr << "Error: " << error << std::endl;
	}
	std::cout << "Usage: "<<me<<" [-B?ilinear] [-C?ubic] [-F?loor] [-L?inear] [-N?earest] [-c?rop? GEOMETRY] [-o OUTPUT_BASE] [-r?eorder? CSV_DIMENSION_INDEX] [-s?cale? CSV_PER_DIMENSION_SCALAR] [-u UNARY_MATH_FUNCTION] <*.cmx>] <CMX...>" << std::endl;
	std::exit(error.empty() ? EXIT_SUCCESS : EXIT_FAILURE);
}

template<typename T> T strtot(const char *str, char ** endptr, const T& dummy);
float strtot(const char *str, char ** endptr, const float& dummy) {return strtof(str, endptr); }
double strtot(const char *str, char ** endptr, const double& dummy) {return strtod(str, endptr); }
long double strtot(const char *str, char ** endptr, const long double& dummy) {return strtold(str, endptr); }
long strtot(const char *str, char ** endptr, const long& dummy) {return strtol(str, endptr, 10); }
long long strtot(const char *str, char ** endptr, const long long& dummy) {return strtoll(str, endptr, 10); }
unsigned long strtot(const char *str, char ** endptr, const unsigned long& dummy) {return strtoul(str, endptr, 10); }
unsigned long long strtot(const char *str, char ** endptr, const unsigned long long& dummy) {return strtoull(str, endptr, 10); }

template<typename T>
static void csv_overwrite(const char *csv, T *array, size_t narray) {
	const char *next = csv;
	for (size_t i=0; i < narray; i++) {
		T n = strtot(next, (char **)&next, *array);
		assert(next != NULL);
		array[i] = n;

		if (*next == '\0') return;
		else next++;
	}
}

int main(int argc, char *argv[]) {
	me = argv[0];
	af::setBackend(AF_BACKEND_CPU); // lots of RAM

	int first_opt = 1, first_cmx=argc;
	for (int i=first_opt; i<argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
				case 'o':
				case 'c':
				case 'r':
				case 's':
				case 'u':
					i++;
					break;
				default:
					break;
			}
		} else {
			first_cmx = i;
			break;
		}
	}

	for (int i=first_cmx; i<argc; i++) {
		double scalars[2] = {1.0, 1.0};
		af_interp_type interpolator = AF_INTERP_NEAREST;
		std::string output_without_cmx_suffix = "";
		af_utils::Geometry crop;
		unsigned long order[4] = {0,1,2,3};

		const char *filename = argv[i];
		std::cout << filename << std::endl;
		af::array A = af_utils::load_cmx(filename);

		for (int o=first_opt; o<first_cmx; o++) {
			switch (argv[o][1]) {
				case 'B': interpolator = AF_INTERP_BILINEAR; break;
				case 'C': interpolator = AF_INTERP_CUBIC; break;
				case 'F': interpolator = AF_INTERP_LOWER; break;
				case 'L': interpolator = AF_INTERP_LINEAR; break;
				case 'N': interpolator = AF_INTERP_NEAREST; break;

				case 'h':
				case '?':
					die();

				default:
					assert(first_cmx-o >= 2);
					switch (argv[o][1]) {
						case 'o': output_without_cmx_suffix = std::string(argv[++o]); break;

						case 'c':
							A = af_utils::crop(A, af_utils::Geometry(argv[++o]));
							break;
						case 'r':
							csv_overwrite(argv[++o], order, sizeof(order)/sizeof(*order));
							A = af::reorder(A, order[0], order[1], order[2], order[3]);
							break;
						case 's':
							csv_overwrite(argv[++o], scalars, sizeof(scalars)/sizeof(*scalars));
							A = af::scale(A, scalars[0], scalars[1], 0, 0, interpolator);
							break;
						case 'u':
							o++;
							if (!strcmp(argv[o], "abs")) {
								A = af::abs(A);
							} else if (!strcmp(argv[o], "exp")) {
								A = af::exp(A);
							} else if (!strcmp(argv[o], "log")) {
								A = af::log(A);
							} else if (!strcmp(argv[o], "log10")) {
								A = af::log10(A);
							} else if (!strcmp(argv[o], "log1p")) {
								A = af::log1p(A);
							} else if (!strcmp(argv[o], "sqrt")) {
								A = af::sqrt(A);
							} else {
								die("Invalid unary math operation \""+std::string(argv[o])+"\"");
							}
							break;
						default:
								 die("Invalid option: \""+std::string(argv[o])+"\"");
					}
			}
		}
		if (output_without_cmx_suffix.empty()) {
			const char *suffix = std::strrchr(filename, '.');
			output_without_cmx_suffix = std::string(filename, suffix);
		}

		af_utils::store_cmx(A, output_without_cmx_suffix);
	}

	return 0;
}
