#!/usr/bin/env python3

from math import isnan, isinf
from os import path
from imp import find_module, load_module
from sys import argv

d,b = path.split(argv[1])
assert(b[-3:] == ".py")
module_tuple = find_module(b[:-3], [d])
module = load_module(b[:-3], *module_tuple)
module_tuple[0].close()

def print_enum(name, members):
    print("enum "+name+" {")
    for m in members:
        print("\t"+m.upper()+",")
    sentinel = "N"+name.upper()
    print("\t"+sentinel)
    print("};")
    print("const std::string %s_name[%s] = {%s};"%(name, sentinel, ",".join(["\""+m+"\"" for m in members])))

def c_type(value):
    if isinstance(value, (list, tuple)):
        return c_type(value[0])
    elif isinstance(value, int):
        return "long long"
    elif isinstance(value, float):
        return "double"
    elif isinstance(value, str):
        return "std::string"
    else:
        raise NotImplementedError("unsupported type")
def c_value(value):
    if isinstance(value, (list, tuple)):
        return "{" + ", ".join([c_value(v) for v in value]) + "}"
    elif isinstance(value, str):
        return "\""+sv.replace("\"", "\\\"")+"\""
    elif isinstance(value, float):
        if isnan(value):
            return "NAN"
        elif isinf(value):
            return ("-" if value < 0 else "") + "INFINITY"
        else:
            return str(value)
    else:
        return str(value)
def c_identifier(key, value):
    identifer = key
    if isinstance(value, (list, tuple)):
        identifer += "["+str(len(value))+"]"
    return identifer

print("#pragma once");
print("");
print("#include <string>");
print("");
for k,v in module.__dict__.items():
    if k != k.upper():
        continue
    print("const "+c_type(v)+" "+c_identifier(k,v)+" = "+c_value(v)+";")

for k,v in module.__dict__.items():
    if k != k.lower() or not isinstance(v, list):
        continue

    print("")
    print_enum(k, v)
