#include <cassert>
#include <cstdlib>
#include <cstring>
#include <string>
#include <arrayfire.h>

#include "../af_utils.hpp"

double saturation = 0.5;
double output_scalar_x = 1.0, output_scalar_y = 1.0;

char *me;
void die(const std::string &error = "") {
	if (!error.empty()) {
		std::cerr << "Error: " << error << std::endl;
	}
	std::cout << "Usage: "<<me<<" [-s SATURATION] [-x OUTPUT_SCALAR_X] [-y OUTPUT_SCALAR_Y] <*.cmx>" << std::endl;
	std::exit(error.empty() ? EXIT_SUCCESS : EXIT_FAILURE);
}

int main(int argc, char *argv[]) {
	me = argv[0];
	af::setBackend(AF_BACKEND_CPU); // lots of RAM

	while (--argc >= 0 && **(++argv) == '-') {
		switch(argv[0][1]) {
			case 'h':
			case '?':
				die();
			default:
				assert(argc >= 2);
				switch(argv[0][1]) {
					case 's': saturation = strtod(*(--argc,++argv), NULL); break;
					case 'x': output_scalar_x = strtod(*(--argc,++argv), NULL); break;
					case 'y': output_scalar_y = strtod(*(--argc,++argv), NULL); break;

					default:
							 die("Invalid option: \""+std::string(argv[0])+"\"");
				}
		}
	}
	if (argc != 1) die("Require arguments");

	char *filename = argv[0];
	af::array A = af_utils::nonInfNaN(af_utils::load_cmx(filename));

	assert(A.dims(2) == 1);
	assert(A.dims(3) == 1);

	af::array intensity = abs(A);
	af::array hue_radians = atan2(imag(A), real(A));

	if (output_scalar_x != 1.0 && output_scalar_y != 1.0) {
		intensity = af::scale(intensity, output_scalar_y, output_scalar_x);
		hue_radians = af::scale(hue_radians, output_scalar_y, output_scalar_x);
	}

	af::array hsv(af::dim4(intensity.dims(0), intensity.dims(1), 3));
	hsv(af::span, af::span, 0) = (1/M_PI) * (hue_radians+M_PI_2);
	hsv(af::span, af::span, 1) = constant(saturation, af::dim4(intensity.dims(0), intensity.dims(1)));
	hsv(af::span, af::span, 2) = af_utils::normalize(af_utils::clampStatistically(intensity, NAN));

	char *suffix = std::strrchr(filename, '.');
	std::string outputName = std::string(filename, suffix) + ".tiff";
	af::saveImage(outputName.c_str(), af::hsv2rgb(hsv));

	return 0;
}
