# The Streaming Pipeline Benchmark described in Section 4.2
A data-intensive benchmark that extracts statistical features from polarized
intensity-only SAR data.

## Implementations
* ArrayFire CUDA: `worker_af_cuda`
* ArrayFire Python: `worker_af.py`
* CUDA: `worker_static`
* MATLAB: `worker_matlab.sh`
* MATLAB (CPU 16 cores): `worker_matlab_cpu.sh`
* PyCUDA: `worker_cuda.py`
* OpenACC: `worker_openacc`
* Thrust: `worker_thrust`
* VexCL OpenCL: `worker_vexcl_cl`
* ArrayFire OpenCL: `worker_af_cl`
* OpenMP (CPU 16 cores): `worker_openmp`

## Running benchmarks
`./$WORKER <CMX>...` 

Where `$WORKER` is the implementation and `<CMX>...` are polarized SAR data in
.cmf32 files that all have identical dimensions. The experiments used the scene
`S1A_IW_GRDH_1SDV_20160429T141541_20160429T141610_011037_01099F_6340` from
[](https://scihub.copernicus.eu/). That data was transformed into cmX files by:

1. Calibrating to BEAM-DIMAP (import into [SNAP](http://step.esa.int/main/toolboxes/snap/),
   then Radio -> Radiometric -> Calibrate, and Run)
2. Determining dimensions (e.g. samples and lines from `.hdr` in BEAM-DIMAP)
3. Given little endian architecture (e.g.~x86), swap endian for each `*.img`
   (e.g. `objcopy -F binary --reverse-byte=4 <IN> <OUT>`)
4. Rename every `*.img` into `*.cmf32_<NCOLUMN>x<NROW>`
5. Optional, transpose cmX files (`cmx_transform -r 1,0 *.cmf32*`)

## Dependencies
* CUDA Toolkit v7.5
* ArrayFire v3.3.1 (CUDA, 64-bit Linux, build f53efc3)
* VexCL 1.3.3 (only `worker_vexcl*`)
* PGC 16.3 or other OpenACC compiler (only `worker_openacc`)
* MATLAB 2015b (only `worker_matlab*`)
* Python 3.4.1 (only `worker_*.`)
	* pycuda 2016.1 (only `worker_cuda.py`)
	* arrayfire 3.3.20160328 (only `worker_af.py`)
