#include <arrayfire.h>

#include "backend.hpp"
#include "common/af_utils.hpp"
#include "config.hpp"

using namespace af;

void compute_products(float **polarizations, size_t h, size_t w, size_t w_before, size_t w_after, float **products) {
	const array smooth_x = constant(1.0/WINDOW_W, WINDOW_W), smooth_y = constant(1.0/WINDOW_H, WINDOW_H);
	const seq without_padding(w_before, w_before+w-1);

	array mli_cov_determinant = constant(1.0, h,w_before+w+w_after);
	array mean_mahalanobis = constant(0.0, h,w_before+w+w_after);
	array sli[NPOLARIZATION];
	array mli[NPOLARIZATION];
	for (int i=0; i<NPOLARIZATION; i++) {
		sli[i] = array(dim4(h,w_before+w+w_after), reinterpret_cast<float*>(polarizations[i]));
		mli[i] = convolve(smooth_y, smooth_x, sli[i]); // NOTE: API stats col,row filter, but convolves along 0th,1st dimension

		mli[i](span,without_padding).host(reinterpret_cast<float*>(products[i]));

		mli_cov_determinant *= mli[i];
		mean_mahalanobis += sli[i] / mli[i];
	}

	array log_rk = log((1.0/(NPOLARIZATION*(NPOLARIZATION+2)))*convolve(smooth_y, smooth_x, mean_mahalanobis*mean_mahalanobis));
	log_rk(span,without_padding).host(reinterpret_cast<float*>(products[LOG_RK]));
	array log_mrcs = 1.0/NPOLARIZATION * log(real(mli_cov_determinant));
	log_mrcs(span,without_padding).host(reinterpret_cast<float*>(products[LOG_MRCS]));
	array log_xpr = log(mli[VH]) - log_mrcs;
	log_xpr(span,without_padding).host(reinterpret_cast<float*>(products[LOG_XPR]));
}
