#include "backend.hpp"
#include "config.hpp"
#include "common/common.cuh"
#include "kernels.cuh"

void compute_products(float **polarizations, size_t h, size_t w, size_t w_before, size_t w_after, float **products) {
	const dim3 din(h,w_before+w+w_after);
	const size_t nin = din.x*din.y, nout = w*h;

	struct polArray dsli, dmli;
	float *dsmoothed_columns;
	CUDA_DIE_UNLESS_SUCCESS(cudaMalloc(&dsmoothed_columns, nin*sizeof(*dsmoothed_columns)));

	for (int i=0; i<NPOLARIZATION; i++) {
		CUDA_DIE_UNLESS_SUCCESS(cudaMalloc(&dsli.d[i], nin*sizeof(**dsli.d)));
		CUDA_DIE_UNLESS_SUCCESS(cudaMemcpyAsync(dsli.d[i], polarizations[i], nin*sizeof(**dsli.d), cudaMemcpyHostToDevice));
		CUDA_DIE_UNLESS_SUCCESS(cudaMalloc(&dmli.d[i], nin*sizeof(**dmli.d)));

		smooth_colwise<<<gs(dim3(nin),BS_1D),BS_1D>>>(dsmoothed_columns, dsli.d[i], din);
		smooth_rowwise<<<gs(dim3(nin),BS_1D),BS_1D>>>(dmli.d[i], dsmoothed_columns, din);

		CUDA_DIE_UNLESS_SUCCESS(cudaMemcpyAsync(products[i], dmli.d[i] + w_before*h, nout*sizeof(**dmli.d), cudaMemcpyDeviceToHost));
	}

	float *dmean_mahalanobis=dsmoothed_columns;
	compute_mean_mahalanobis<<<gs(dim3(nin),BS_1D),BS_1D>>>(dmean_mahalanobis, dsli, dmli, din);
	float *dmean_mahalanobis_colwise_smoothed=dsli.d[0];
	smooth_colwise<<<gs(dim3(nin),BS_1D),BS_1D>>>(dmean_mahalanobis_colwise_smoothed, dmean_mahalanobis, din);
	smooth_rowwise<<<gs(dim3(nin),BS_1D),BS_1D>>>(dmean_mahalanobis, dmean_mahalanobis_colwise_smoothed, din);

	float *dlog_rk = dsmoothed_columns, *dlog_mrcs = dsli.d[0], *dlog_xpr = dsli.d[1];

	compute_properties<<<gs(dim3(nin),BS_1D),BS_1D>>>(dlog_rk, dlog_mrcs, dlog_xpr, dmli, dmean_mahalanobis, din);

	CUDA_DIE_UNLESS_SUCCESS(cudaMemcpyAsync(products[LOG_RK], dlog_rk + w_before*h, nout*sizeof(*dlog_rk), cudaMemcpyDeviceToHost));
	CUDA_DIE_UNLESS_SUCCESS(cudaMemcpyAsync(products[LOG_MRCS], dlog_mrcs + w_before*h, nout*sizeof(*dlog_mrcs), cudaMemcpyDeviceToHost));
	CUDA_DIE_UNLESS_SUCCESS(cudaMemcpyAsync(products[LOG_XPR], dlog_xpr + w_before*h, nout*sizeof(*dlog_xpr), cudaMemcpyDeviceToHost));

	for (int i=0; i<NPOLARIZATION; i++) {
		CUDA_DIE_UNLESS_SUCCESS(cudaFree(dmli.d[i]));
		CUDA_DIE_UNLESS_SUCCESS(cudaFree(dsli.d[i]));
	}
	CUDA_DIE_UNLESS_SUCCESS(cudaFree(dsmoothed_columns));
	CUDA_DIE_UNLESS_SUCCESS(cudaThreadSynchronize());
}
