#include <cassert>
#include <cerrno>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <string>

#include <arrayfire.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <unistd.h>

#include "backend.hpp"
#include "common/af_utils.hpp"
#include "config.hpp"

using namespace std;

enum named_args {
	EXEC,
	H,
	W,
	W_BEFORE,
	W_AFTER,
	X,
	NARG_BEFORE_FILES
};

char *output_prefix = NULL;

char *me;
static void die(const std::string &error = "") {
	if (!error.empty()) {
		std::cerr << "Error: " << error << std::endl;
	}
	std::cout << "Usage: "<<me<<"[-o OUTPUT_PREFIX] <WORKER> <POLARIZATION_1.cmx> ... <POLARIZATION_N.cmx>" << std::endl;
	std::exit(error.empty() ? EXIT_SUCCESS : EXIT_FAILURE);
}

int main(int argc, char *argv[]) {
	me = argv[0];

	while (--argc > 0 && **(++argv) == '-') {
		switch(argv[0][1]) {
			case 'h':
			case '?':
				die();

			default:
				assert(argc >= 2);
				switch(argv[0][1]) {
					case 'o': output_prefix = (*(--argc,++argv)); break;

					default:
							 die("Invalid option: \""+std::string(argv[0])+"\"");
				}
		}
	}
	if (argc < 1) die("Require "+std::to_string(1+NPOLARIZATION)+" arguments");
	char *newargs[NARG_BEFORE_FILES + NPOLARIZATION + NPRODUCT + 1] = {(char*)0};
	newargs[EXEC] = *(argc--,argv++);
	for (int i=EXEC+1; i < NARG_BEFORE_FILES; i++) {
		newargs[i] = (char *)alloca(22); // enough to fit SIZE_T as decimal
		assert(newargs[i] != NULL);
	}

	af::dim4 dims = af_utils::dims_from_cmx(argv[0]);
	for (int i=0; i<NPOLARIZATION; i++) {
		assert(af_utils::dims_from_cmx(argv[i]) == dims);
		assert(af_utils::dtype_from_cmx(argv[i]) == f32);
		newargs[NARG_BEFORE_FILES+i] = argv[i];
	}

	const dim_t h=dims[0], w=dims[1];
	assert(dims[2] == 1);
	assert(dims[3] == 1);

	const dim_t nchunk = w/NCOL_PER_CHUNK + (w%NCOL_PER_CHUNK != 0);

	if (output_prefix) {
		for (int i=0; i < NPRODUCT; i++) {
			std::string product = std::string(output_prefix) + "_" + product_name[i] + af_utils::cmx_suffix(f32, dims);

			int this_arg = NARG_BEFORE_FILES + NPOLARIZATION + i;
			newargs[this_arg] = (char *)alloca(product.length() + 1);
			assert(newargs[this_arg] != NULL);
			std::strcpy(newargs[this_arg], product.c_str());

			int fd = open(newargs[this_arg], O_RDWR|O_TRUNC|O_CREAT, 0666);
			assert(fd >= 0);
			lseek(fd, dims.elements()*sizeof(float)-1, SEEK_SET);
			char a_byte = 0;
			write(fd, &a_byte, 1);
			assert(close(fd) == 0);
		}
	}

	std::sprintf(newargs[H], "%llu", h);
	for (dim_t i=0; i<nchunk; i++) {
		std::cout << i << "/" << nchunk << std::endl;
		const dim_t x = i*NCOL_PER_CHUNK;
		const dim_t w_before = std::min(x, (2*WINDOW_W/2)) * (i > 0); // NOTE: twice the padding to envelop LOG_RK's smoothing of smoothed MLI
		const dim_t w_after = x+NCOL_PER_CHUNK >= w ? 0 : std::min(w-x-NCOL_PER_CHUNK, (2*WINDOW_W/2));
		const dim_t w_input = std::min(NCOL_PER_CHUNK, w-w_after-x);

		std::sprintf(newargs[W], "%llu", w_input);
		std::sprintf(newargs[W_BEFORE], "%llu", w_before);
		std::sprintf(newargs[W_AFTER], "%llu", w_after);
		std::sprintf(newargs[X], "%llu", x);

		if (fork() == 0) {
			if(execv(newargs[EXEC], (char *const*)newargs) != 0) {
				die("execv failed with: "+std::to_string(errno));
			}
		} else {
			int status;
			wait(&status);
		}
	}
	return 0;
}
