#!/usr/bin/env python3
from common import cmx
from config import *
from ctypes import cast, c_char, c_float, c_void_p, sizeof
from mmap import mmap, MAP_SHARED, PROT_READ, PROT_WRITE
from optparse import OptionParser
from sys import stdout
from time import clock

import arrayfire as af
import os
import math

def orchestrate(compute_features_callback):
    parser = OptionParser()
    parser.add_option("-o", "--ouput-prefix", dest="output_prefix", help="if set, PREFIX for product output files", metavar="PREFIX")
    (options, args) = parser.parse_args()

    sli_fd = []; product_fd = []
    sli_mmap = []; product_mmap = []
    try:
        (dtype, dims) = cmx.parse(args[0])
        h,w = dims
        PADDING = 2*int(WINDOW_W/2)
        NBYTE = sizeof(af.to_c_type[dtype.value])

        sli_fd = [os.open(fn, os.O_RDWR) for fn in args]
        if options.output_prefix:
            product_fd = [os.open(options.output_prefix+"_"+p+cmx.suffix(dtype, dims), os.O_RDWR|os.O_TRUNC|os.O_CREAT) for p in product]
            for i in range(len(product_fd)):
                os.lseek(product_fd[i], NBYTE*h*w-1, os.SEEK_SET)
                os.write(product_fd[i], bytearray(1))
                product_mmap.append(mmap(product_fd[i], 0, MAP_SHARED, PROT_READ|PROT_WRITE))
        else:
            product_mmap = [bytearray(NBYTE*h*NCOL_PER_CHUNK) for ignore in product]

        sli_mmap = [mmap(fd, 0, MAP_SHARED, PROT_READ|PROT_WRITE) for fd in sli_fd]

        nchunk = math.ceil(w/NCOL_PER_CHUNK)
        for i in range(nchunk):
            stdout.write("%d/%d %f "%(i, nchunk, clock()))
            x = i*NCOL_PER_CHUNK
            w_before = 0 if x == 0 else min(PADDING, x)
            w_after = 0 if x+NCOL_PER_CHUNK >= w else min(PADDING, w-x-NCOL_PER_CHUNK)
            w_chunk = min(NCOL_PER_CHUNK, w-w_after-x)

            hsli = [(c_float*h*(w_before+w_chunk+w_after)).from_buffer(m, h*(x-w_before)*NBYTE) for m in sli_mmap]
            hproduct = [(c_float*h*w_chunk).from_buffer(m, h*x*NBYTE if options.output_prefix else 0) for m in product_mmap]
            compute_features_callback(hsli, h, w_chunk, w_before, w_after, hproduct)
            stdout.write("%f\n"%(clock()))

    finally:
        for m in sli_mmap:
            m.close()
        if options.output_prefix:
            for m in product_mmap:
                m.close()
        for fd in sli_fd + product_fd:
            os.close(fd)
