function [] = sli_features(sli_fns, product_prefix)
	tic;
	P = config;

	[type,dims] = cmxExtract(sli_fns{1});

	sli_mmap = {};
	for i=1:numel(sli_fns)
		sli_mmap{i} = memmapfile(sli_fns{i}, 'Format', {'single', dims, 'm'});
	end

	product_fds = {};
	if numel(product_prefix) ~= 0
		for i=1:numel(P.product)
			fn = [product_prefix, '_', P.product{i}, cmxCreate('f32', dims)];
			product_fds{i} = fopen(fn, 'w'); % NOTE: MATLAB mmap does not support writing GPUArrays directly
		end
	end

	PADDING = floor(P.WINDOW_W/2)*2;
	nchunk = ceil(dims(2) / P.NCOL_PER_CHUNK);
	for i = 0:nchunk-1
		fprintf('STDOUT %i/%i %f ', i, nchunk, toc);
		x = i*P.NCOL_PER_CHUNK;
		w_before = min([PADDING, x]);
		w_after = min([PADDING, dims(2)-x-P.NCOL_PER_CHUNK]);
		if i == 0
			w_before = 0;
		end
		if x+P.NCOL_PER_CHUNK >= dims(2);
			w_after = 0;
		end
		w_chunk = min([P.NCOL_PER_CHUNK, dims(2)-w_after-x]);
		compute_chunk(dims(1), w_chunk, w_before, w_after, x, sli_mmap, product_fds, P)
		fprintf('%f\n', toc);
	end
	for i=1:numel(product_fds)
		fclose(product_fds{i});
	end
end

function [] = compute_chunk(h, w, w_before, w_after, x, sli_mmap, product_fds, P)
	WINDOW = [P.WINDOW_H,P.WINDOW_W];
	averaging = ones(WINDOW)./prod(WINDOW);

	npol = numel(sli_mmap);
	w_in = w_before + w + w_after;

	sli = zeros(h, w_in, npol, 'gpuArray');
	EPF = zeros(h, w_in, npol+3, 'gpuArray');
	mli_cov_determinant = ones(h, w_in, 'gpuArray');
	mean_mahalanobis = zeros(h, w_in, 'gpuArray');
	for i=1:npol
		sli(:,:,i) = sli_mmap{i}.Data.m(:,x-w_before+1:(x+w+w_after));

		EPF(:,:,i) = imfilter(sli(:,:,i), averaging, 'conv'); % MLI(alphabetically)
		mli_cov_determinant = mli_cov_determinant.*EPF(:,:,i);
		mean_mahalanobis = mean_mahalanobis + sli(:,:,i)./EPF(:,:,i);
	end

	EPF(:,:,3) = log(1/(npol*(npol+2)) .* imfilter(mean_mahalanobis.*mean_mahalanobis, averaging)); % logRk
	EPF(:,:,4) = 1/npol * real(log(complex(mli_cov_determinant))); % logMrcs
	EPF(:,:,5) = log(EPF(:,:,1)) - EPF(:,:,4); % logXpr

	hEPF = gather(EPF); % when mmap is not a possibility fetching everything in one go is faster
	for i=1:numel(product_fds)
		fwrite(product_fds{i}, hEPF(:, w_before+1:w_before+w, i), 'float32');
	end
end
