#include "backend.hpp"
#include "config.hpp"
#include "common/common.cuh"
#include "kernels.cuh"

#include <thrust/device_vector.h>
#include <thrust/functional.h>

using namespace thrust;
using namespace thrust::placeholders;

#define PTR(dvector) raw_pointer_cast(dvector.data())

template<typename BOp, typename T> struct t2r: public unary_function<tuple<T,T>, T> {
	__host__ __device__ T operator()(tuple<T,T> const t) const { return BOp()(get<0>(t), get<1>(t)); }
};
template<int Divisor, typename T> struct logd: public unary_function<T, T> {
	__host__ __device__ T operator()(const T n) const { return log(n/Divisor); }
};
template<int Divisor, typename T> struct dlog: public unary_function<T, T> {
	__host__ __device__ T operator()(const T n) const { return log(n)/Divisor; }
};
template<typename T> struct loga_minus_b: public binary_function<T, T, T> {
	__host__ __device__ T operator()(const T a, const T b) const { return log(a) - b; }
};

void compute_products(float **polarizations, size_t h, size_t w, size_t w_before, size_t w_after, float **products) {
	const dim3 din(h,w_before+w+w_after);
	const size_t nin = din.x*din.y;

	device_vector<float> mli[NPOLARIZATION], smoothed_columns(nin), sqrt_mean_mahalanobis(nin, .0f), cov_determinant(nin, 1.f);

	for (int i=0; i<NPOLARIZATION; i++) {
		device_vector<float> sli(polarizations[i], polarizations[i]+nin);
		smooth_colwise<<<gs(nin,BS_1D),BS_1D>>>(PTR(smoothed_columns), PTR(sli), din);
		mli[i] = device_vector<float>(nin);
		smooth_rowwise<<<gs(nin,BS_1D),BS_1D>>>(PTR(mli[i]), PTR(smoothed_columns), din);

		auto sli_over_mli = make_transform_iterator(make_zip_iterator(make_tuple(sli.begin(),mli[i].begin())), t2r<divides<float>,float>());
		transform(sqrt_mean_mahalanobis.begin(), sqrt_mean_mahalanobis.end(), sli_over_mli, sqrt_mean_mahalanobis.begin(), plus<float>());

		transform(cov_determinant.begin(), cov_determinant.end(), mli[i].begin(), cov_determinant.begin(), multiplies<float>());

		copy_n(mli[i].begin() + h*w_before, h*w, products[i]);
	}

	transform(sqrt_mean_mahalanobis.begin(), sqrt_mean_mahalanobis.end(), sqrt_mean_mahalanobis.begin(), mli[VV].begin(), multiplies<float>());
	smooth_colwise<<<gs(nin,BS_1D),BS_1D>>>(PTR(smoothed_columns), PTR(mli[VV]), din);
	smooth_rowwise<<<gs(nin,BS_1D),BS_1D>>>(PTR(mli[VV]), PTR(smoothed_columns), din);
	for_each(mli[VV].begin(), mli[VV].end(), logd<NPOLARIZATION*(NPOLARIZATION+2),float>());
	copy_n(mli[VV].begin() + h*w_before, h*w, products[LOG_RK]);

	transform(cov_determinant.begin(), cov_determinant.end(), mli[VV].begin(), dlog<NPOLARIZATION,float>());
	copy_n(mli[VV].begin() + h*w_before, h*w, products[LOG_MRCS]);

	transform(mli[VH].begin(), mli[VH].end(), mli[VV].begin(), mli[VV].begin(), loga_minus_b<float>());
	copy_n(mli[VV].begin() + h*w_before, h*w, products[LOG_XPR]);
}
