#include <iostream>
#include <vexcl/vexcl.hpp>

#include "backend.hpp"
#include "config.hpp"

vex::Context ctx(vex::Filter::Position(0));

VEX_CONSTANT(MM_DIVISOR, float(NPOLARIZATION*(NPOLARIZATION+2)));
VEX_CONSTANT(LOG_MRCS_DIVISOR, float(NPOLARIZATION));

VEX_FUNCTION(float, smooth, (const float *,in)(size_t,WINDOW)(size_t,stride)(size_t,ndim)(size_t,i),
	ulong j = (i/stride) % ndim;

	ulong nbelow = WINDOW/2 + (WINDOW%2==0), nabove = WINDOW/2;
	if (j-nbelow >= ndim) {nbelow = j;} // NOTE: underflow
	if (j+nabove >= ndim) {nabove = ndim-1-j;}

	float sum = 0.0;
	for (size_t ii=i-stride*nbelow; ii <= i+stride*nabove; ii+=stride) {
		sum = sum + in[ii];
	}
	return sum / (float)nbelow+1+nabove;
);

void compute_products(float **polarizations, size_t h, size_t w, size_t w_before, size_t w_after, float **products) {
	size_t win = w_before+w+w_after;
	size_t nin = h*win;

	vex::vector<float> mli[NPOLARIZATION], sqrt_mean_mahalanobis(ctx,nin), cov_determinant(ctx,nin), sli(ctx,nin), smoothed_columns(ctx,nin);
	sqrt_mean_mahalanobis = 0;
	cov_determinant = 1;

	for (int i=0; i<NPOLARIZATION; i++) {
		sli.write_data(0, nin, polarizations[i], true);

		smoothed_columns = smooth(vex::raw_pointer(sli), size_t(WINDOW_H), 1UL, h, vex::element_index());
		mli[i] = vex::vector<float>(ctx, nin);
		mli[i] = smooth(vex::raw_pointer(smoothed_columns), size_t(WINDOW_W), h, w, vex::element_index());
		mli[i].read_data(h*w_before, h*w, products[i], false);

		sqrt_mean_mahalanobis = sqrt_mean_mahalanobis + sli/mli[i];
		cov_determinant = cov_determinant + mli[i];
	}

	vex::vector<float> mean_mahalanobis = sqrt_mean_mahalanobis*sqrt_mean_mahalanobis;
	smoothed_columns = smooth(vex::raw_pointer(mean_mahalanobis), size_t(WINDOW_H), 1UL, h, vex::element_index());
	vex::vector<float> smoothed_mm(nin);
	smoothed_mm = smooth(vex::raw_pointer(smoothed_columns), size_t(WINDOW_W), h, w, vex::element_index());
	vex::vector<float> log_rk = log(smoothed_mm/MM_DIVISOR());
	log_rk.read_data(h*w_before, h*w, products[LOG_RK], false);
	vex::vector<float> log_mrcs = log(cov_determinant)/LOG_MRCS_DIVISOR();
	log_mrcs.read_data(h*w_before, h*w, products[LOG_MRCS], false);
	vex::vector<float> log_xpr = log(mli[VH])-log_mrcs;
	log_xpr.read_data(h*w_before, h*w, products[LOG_XPR], false);
}
