#!/usr/bin/env python3
from config import *
from ctypes import cast, c_void_p, memmove, sizeof
from orchestrator import orchestrate

import arrayfire as af

def compute_features(hsli, h, w, w_before, w_after, hproduct):
    sli = [af.Array(cast(b, c_void_p), (h, w_before+w+w_after), af.Dtype.f32) for b in hsli]

    smooth_x = af.constant(1.0/WINDOW_W, WINDOW_W); smooth_y = af.constant(1.0/WINDOW_H, WINDOW_H)
    mli_cov_determinant = af.constant(1.0, h, w_before+w+w_after)
    mean_mahalanobis = af.constant(0.0, h, w_before+w+w_after)

    mli = [0]*len(sli)
    for i in range(0,len(sli)):
        mli[i] = af.convolve2_separable(smooth_y, smooth_x, sli[i])

        mli_cov_determinant *= mli[i]
        mean_mahalanobis += sli[i]/mli[i]

    log_rk = af.log(af.convolve2_separable(smooth_y, smooth_x, mean_mahalanobis*mean_mahalanobis)/(len(mli)*(len(mli)+2)))
    log_mrcs = af.log(mli_cov_determinant)/len(mli)
    log_xpr = af.log(mli[polarization.index("vh")])-log_mrcs

    for i in range(0,len(sli)):
        memmove(hproduct[i], mli[i][:,w_before:w_before+w].to_ctype(), h*w*sizeof(af.to_c_type[mli[i].type()]))
    memmove(hproduct[product.index("log_rk")], log_rk[:,w_before:w_before+w].to_ctype(), h*w*sizeof(af.to_c_type[log_rk.type()]))
    memmove(hproduct[product.index("log_mrcs")], log_mrcs[:,w_before:w_before+w].to_ctype(), h*w*sizeof(af.to_c_type[log_mrcs.type()]))
    memmove(hproduct[product.index("log_xpr")], log_xpr[:,w_before:w_before+w].to_ctype(), h*w*sizeof(af.to_c_type[log_xpr.type()]))

orchestrate(compute_features)
