#!/usr/bin/env python3
from common.cuda import dim3, gs, TS_1D
from config import *
from ctypes import memmove, sizeof
from orchestrator import orchestrate
from pycuda.compiler import SourceModule

import numpy as np
import pycuda.cumath as dm
import pycuda.driver as cuda
import pycuda.autoinit # noqa
import pycuda.gpuarray as da
import os

mod = None
with open("kernels.cuh", "r") as f:
    mod = SourceModule(f.read(), no_extern_c=1, include_dirs=[os.path.dirname(os.path.abspath(__file__))])
smooth_colwise = mod.get_function('smooth_colwise')
smooth_rowwise = mod.get_function('smooth_rowwise')

def smooth(dst, src, h, w, tmp):
    smooth_colwise(tmp.gpudata, src.gpudata, dim3(h,w), grid=gs((h*w,1,1),TS_1D), block=TS_1D)
    smooth_rowwise(dst.gpudata, tmp.gpudata, dim3(h,w), grid=gs((h*w,1,1),TS_1D), block=TS_1D)

def compute_features(hsli, h, w, w_before, w_after, hproduct):
    w_in = w_before + w + w_after
    gsli = [cuda.mem_alloc(sizeof(ha)) for ha in hsli]
    for i in range(len(hsli)):
        cuda.memcpy_htod(gsli[i], hsli[i])
    sli = [da.GPUArray((h,w_in), np.float32, gpudata=gsli[i], order="F") for i in range(len(gsli))]
    tmp = da.empty((h,w_in), np.float32, order="F")

    mli = [da.empty((h,w_in), np.float32, order="F") for ignore in gsli]
    mli_cov_determinant = da.to_gpu(np.ones((h,w_in), np.float32, order="F"))
    mean_mahalanobis = da.zeros((h,w_in), np.float32, order="F")
    for i in range(len(hsli)):
        smooth(mli[i], sli[i], h, w_in, tmp)
        cuda.memcpy_dtoh_async(hproduct[i], mli[i][:,w_before:w_before+w].gpudata)

        mli_cov_determinant *= mli[i]
        mean_mahalanobis += sli[i] / mli[i]

    mean_mahalanobis *= mean_mahalanobis
    log_rk = da.empty((h,w_in), np.float32, order="F")
    smooth(log_rk, mean_mahalanobis, h, w_in, tmp)
    log_rk = dm.log(log_rk / len(polarization)*(len(polarization)+2))
    cuda.memcpy_dtoh_async(hproduct[product.index("log_rk")], log_rk[:,w_before:w_before+w].gpudata)
    log_mrcs = dm.log(mli_cov_determinant) / len(polarization)
    cuda.memcpy_dtoh_async(hproduct[product.index("log_mrcs")], log_mrcs[:,w_before:w_before+w].gpudata)
    log_xpr = dm.log(mli[polarization.index("vh")])-log_mrcs
    cuda.memcpy_dtoh_async(hproduct[product.index("log_xpr")], log_xpr[:,w_before:w_before+w].gpudata)
    cuda.Context.synchronize()

orchestrate(compute_features)
