#pragma once
#include <cstring>
#include <cuda_runtime.h>

static inline void cuda_timer_copy_non_nil(float *dst, void *src) {
	if (dst != NULL) *dst = *((float *)src);
}
#define CUDA_TIMER(prefix, cuda_stream, out_ms) \
	for (\
			cudaEvent_t \
				cuda_timer_start=NULL, cuda_timer_stop=NULL\
				,cuda_timer_stream = (cudaEvent_t)(cuda_stream) /*CUDA types are mostly pointers, so they can be casted freely*/\
				,cuda_timer_ms = NULL /* pointers are larger than floats, so exploiting its first 4 bytes as such*/\
		;\
			    cuda_timer_start==NULL /* escape after first loop */\
			&& !cudaEventCreate(&cuda_timer_start)\
			&& !cudaEventCreate(&cuda_timer_stop)\
			&& !cudaEventRecord(cuda_timer_start, (cudaStream_t)cuda_timer_stream)\
		;\
			   !cudaEventRecord(cuda_timer_stop, (cudaStream_t)cuda_timer_stream)\
			&& !cudaEventSynchronize(cuda_timer_stop)\
			&& !cudaEventElapsedTime((float *)&cuda_timer_ms, cuda_timer_start, cuda_timer_stop)\
			&& !cudaEventDestroy(cuda_timer_start)\
			&& !cudaEventDestroy(cuda_timer_stop)\
			, (std::strcmp((prefix),"") != 0 && std::cout << (prefix) << *((float*)&cuda_timer_ms)<<"ms" << std::endl)\
			, cuda_timer_copy_non_nil((out_ms), &cuda_timer_ms)\
	)
